@@ -8,24 +8,29 @@ from cl1.classes import utils
 import datetime 
 
 # argument functions.
-def argument_present(arguments):
+def argument_present(arguments, default=False, count=1):
 	if isinstance(arguments, str):
-		if arguments in sys.argv: return True
-		else: return False
+		c = sys.argv.count(arguments)
+		if c > 0 and count <= c:
+			return True
+		else: return default
 	elif isinstance(arguments, list):
 		for argument in arguments:
-			if argument in sys.argv: return True
-		return False
+			c = sys.argv.count(argument)
+			if c > 0 and count <= c:
+				return True
+		return default
 	else: raise ValueError("Invalid usage, arguments must either be a list or string.")
-def arguments_present(arguments):
-	if isinstance(arguments, str): return argument_present(argument)
+def arguments_present(arguments, default=False, count=1):
+	if isinstance(arguments, str): return argument_present(argument, default=default, count=count)
 	else:
 		for argument in arguments:
-			if argument_present(argument):
+			if argument_present(argument, default=default, count=count):
 				return True
-		return False
-def get_argument(argument, required=True, index=1, default=None, ):
-
+		return default
+def get_argument(argument, required=True, index=1, count=1, default=None, ):
+	if default != None and required: required = False
+	
 	# set error msg.
 	if index == 1:
 		empty_error = f"Define argument [{argument}]."
@@ -39,10 +44,13 @@ def get_argument(argument, required=True, index=1, default=None, ):
 		else: return default
 
 	# retrieve.
-	y = 0
+	y, c = 0, 0
 	for x in sys.argv:
 		try:
-			if x == argument: return sys.argv[y+index]
+			if x == argument: 
+				c += 1
+				if count == c:
+					return sys.argv[y+index]
 		except IndexError:
 			if required:
 				raise EmptyArgumentError(empty_error)
@@ -50,7 +58,11 @@ def get_argument(argument, required=True, index=1, default=None, ):
 		y += 1
 
 	# should not happen.
-	return default
+	if required:
+		raise EmptyArgumentError(empty_error)
+	else: return default
+
+	#
 def __str_representable__(dict):
 	str = json.dumps(dict, indent=4, ensure_ascii=False)[:-2][2:].replace('    "', '    ').replace('": "', " : ")
 	s, c, max = "", 0, len(str.split('\n'))
@@ -90,11 +102,11 @@ class CLI(object):
 			self.assign(attributes)
 		def iterate(self):
 			return sys.argv
-		def present(self, argument):
+		def present(self, argument, default=False, count=1):
 			if isinstance(argument, str):
-				return argument_present(argument)
+				return argument_present(argument, default=False, count=count)
 			else:
-				return arguments_present(argument)
+				return arguments_present(argument, default=False, count=count)
 		def get(self, 
 			# the argument id (str) (example: --path).
 			argument, 
@@ -102,6 +114,10 @@ class CLI(object):
 			required=True, 
 			# the plus index.
 			index=1, 
+			# the argument id count index.
+			count=1, 
+			# the value format (str, list) (example: formats=[str].
+			format="*",
 			# pack multiple into tuple.
 			pack=True, 
 			# default value when empty.
@@ -115,6 +131,75 @@ class CLI(object):
 			# overwrite default notes (dict) (specify to use).
 			notes=None,
 		):
+
+			# check & clean format.
+			def clean_string(string="", remove_first_space=False, remove_last_space=False):
+				while True:
+					if string in [str, bool, int, float, dict, list, object, tuple]: break
+					elif remove_first_space and len(string) > 0 and string[0] == " ": string = string[1:]
+					elif remove_last_space and len(string) > 0 and string[len(string)-1] == " ": string = string[:-1]
+					else: break
+				return string
+			def clean_array(array=["x", "y"] or "x,y", remove_first_space=False, remove_last_space=False):
+				if isinstance(array, str): 
+					array = array.split(",")
+				new = []
+				for i in array: new.append(clean_string(i, remove_first_space=remove_first_space, remove_last_space=remove_last_space))
+				return new
+			def clean_dict(dictionary={} or "x:0,y:1"):
+				if isinstance(dictionary, str): dictionary = clean_array(dict.split(","), remove_first_space=True, remove_last_space=True)
+				if isinstance(dictionary, list):
+					new = {}
+					for key in dictionary:
+						new[clean_string(key, remove_first_space=remove_first_space, remove_last_space=remove_last_space)] = clean_string(value, remove_first_space=remove_first_space, remove_last_space=remove_last_space)
+					return new
+				elif isinstance(dictionary, dict):
+					new = {}
+					for key,value in dictionary.items():
+						new[clean_string(key, remove_first_space=remove_first_space, remove_last_space=remove_last_space)] = clean_string(value, remove_first_space=remove_first_space, remove_last_space=remove_last_space)
+					return new
+			if not isinstance(format, list):
+				if format not in [str, bool, int, float, dict, list, object, tuple]:
+					try:
+						format = format.split(",")
+					except: 
+						format = [format]
+				else:
+					format = [format]
+			if "*" not in format:
+				l = []
+				for i in clean_array(format, remove_first_space=True, remove_last_space=True):
+					if i not in [
+						str, "str", "string",
+						bool, "bool", "boolean",
+						int, "int", "integer",
+						float, "float", "double",
+						dict, "dict", "json", "dictionary",
+						list, "list", "array",
+						object, "object", "obj",
+						tuple, "tuple",
+					]:
+						format_error = f"""Specified an invalid format type [{i}], valid formats: {str([str, bool, int, float, tuple, dict, list, object]).replace("'",'')}."""
+						if json:
+							r3sponse.log(error=empty_error, json=True)
+							sys.exit(1)
+						else:
+							self.docs(stop=False, chapter=chapter, mode=mode, notes=notes)
+							sys.tracebacklimit = -1 ; sys.traceback_limit = -1
+							raise InvalidFormatError(invalid_format_error)
+
+					if i in [str, "str", "string"]: i = str
+					if i in [bool, "bool", "boolean"]: i = int
+					if i in [int, "int", "integer"]: i = int
+					if i in [float, "float", "double"]: i = float
+					if i in [dict, "dict", "json", "dictionary"]: i = dict
+					if i in [list, "list", "array"]: i = list
+					if i in [object, "object", "obj"]: i = object
+					if i in [tuple, "tuple"]: i = tuple
+					l.append(i)
+				format = l
+
+			# handle string.
 			if isinstance(argument, str):
 				
 				# set error msg.
@@ -136,12 +221,55 @@ class CLI(object):
 					else: return default
 
 				# retrieve.
-				y = 0
+				y, c = 0, 0
 				for x in sys.argv:
+					# get value.
 					failed = False
 					try:
-						if x == argument: return sys.argv[y+index]
+						if x == argument: 
+							c += 1
+							if c == count:
+								value = sys.argv[y+index]
+								# check format.
+								if "*" not in format:
+									x_format = Formats.get(value, serialize=False)
+									error = False
+									if x_format == str and int in format:
+										try:
+											value = int(value)
+										except: error = True
+									if x_format == str and bool in format:
+										try:
+											if value in ["true", True, "True", "TRUE"]:
+												value = True
+											else: 
+												value = False
+										except: error = True
+									if x_format == str and float in format:
+										try:
+											value = float(value)
+										except: error = True
+									if x_format == str and list in format:
+										try:
+											value = clean_array(value, remove_first_space=True, remove_last_space=True)
+										except: error = True
+									if x_format == str and dict in format:
+										try:
+											value = clean_dict(value, remove_first_space=True, remove_last_space=True)
+										except: error = True
+									x_format = Formats.get(value, serialize=False)
+									if error or x_format not in format:
+										format_error = f"""Provided an incorrect [{argument}] format: [{value}:{x_format}], valid format options: {str(format).replace("'",'')}."""
+										if json:
+											r3sponse.log(error=format_error, json=True)
+											sys.exit(1)
+										else:
+											self.docs(stop=False, chapter=chapter, mode=mode, notes=notes)
+											sys.tracebacklimit = -1 ; sys.traceback_limit = -1
+											raise ArgumentFormatError(format_error)
+								return value
 					except IndexError: failed = True
+					# not present.
 					if failed:
 						if required:
 							if json:
@@ -155,20 +283,28 @@ class CLI(object):
 					y += 1
 
 				# should not happen.
-				return default
+				if required:
+					if json:
+						r3sponse.log(error=empty_error, json=True)
+						sys.exit(1)
+					else:
+						self.docs(stop=False, chapter=chapter, mode=mode, notes=notes)
+						sys.tracebacklimit = -1 ; sys.traceback_limit = -1
+						raise EmptyArgumentError(empty_error)
+				else: return default
 
 			else:
 				if pack:
 					arguments = []
 					for i in argument: 
-						arguments.append(self.get(i, required=required, index=index,default=default, mode=mode, chapter=chapter, notes=notes, json=json))
+						arguments.append(self.get(i, required=required, index=index,default=default, mode=mode, chapter=chapter, notes=notes, json=json, count=count))
 					return arguments	
 				else:
 					for i in argument: 
-						a = self.get(i, required=required, index=index,default=default, mode=mode, chapter=chapter, notes=notes, json=json)
+						a = self.get(i, required=required, index=index,default=default, mode=mode, chapter=chapter, notes=notes, json=json, count=count)
 						if a != default: return a
 					return default
-		def check(self, exceptions=["--log-level", "--create-alias"], json=False):
+		def check(self, exceptions=["--log-level", "--create-alias", "--version", "-j", "--json", "--non-interactive"], json=False):
 			lexecutable = self.executable
 			while True:
 				if len(lexecutable) > 0 and lexecutable[len(lexecutable)-1] == "/": lexecutable = lexecutable[:-1]
@@ -548,6 +684,33 @@ class CLI(object):
 		if n != "": doc += "\nNotes:\n"+n
 		doc += c
 		return doc
-
+	def __str_representable__(self, dict, start_indent=4):
+		str = json.dumps(dict, indent=4, ensure_ascii=False)[:-2][2:].replace('    "', '    ').replace('": "', " : ").replace('": ', " : ")
+		s, c, max = "", 0, len(str.split('\n'))
+		for i in str.split('\n'):
+			if i not in ["", " "]:
+				#if c < max-1:
+				#if (len(i) >= len('",\n') and i[:len('",\n')] == '",\n') or (len(i) >= len(',\n') and i[:len(',\n')] == ',\n') or (len(i) >= len('"\n') and i[:len(',\n')] == '"\n'):
+				#	s += i[:-2]+"\n"
+				#else:
+				if len(i) > 0 and i[len(i)-1] == ',': i = i[:-1]
+				if len(i) > 0 and i[len(i)-1] == '\n': i = i[:-1]
+				if len(i) > 0 and i[len(i)-1] == '"': i = i[:-1]
+				s += i+"\n"
+			c += 1
+		if len(s) > 0 and s[len(s)-1] == "\n": s = s[:-1]
+		if len(s) > 0 and s[len(s)-1] == "}": s = s[:-1]
+		if len(s) > 0 and s[0] == "\n": s = s[1:]
+		if len(s) > 0 and s[0] == "{": s = s[1:]
+		string = s.replace(": : \n"," :\n").replace(" : \n",":\n")
+		if start_indent != 4:
+			newstring, lindent = "", None
+			for i in string.split("\n"):
+				newstring += i[4:]+"\n"
+			string = newstring
+		while True:
+			if len(string) > 0 and string[len(string)-1] == "\n": string = string[:-1]
+			else: break
+		return string.replace(" : {", " :").replace(" :\n", ":\n").replace(" : ", ": ")
 
 #
\ No newline at end of file
